function [DF] = int_DF_mg_chebyshev(a,b,par)

%  Chebyshev integrator for the polynomialization vector field of Mackey-Glass
%
%  inputs: (1) a = (a_0,a_1,a_2,...,a_N) : the Chebyshev coefficients of the solution y(t) in R^4
%		   (2) b = (b_0,b_1,b_2,...,b_N) : the Chebyshev coefficients of the history x(t) = y(t-tau) in R^4
%          (3) par = (gamma,beta,rho,tau) : the parameters

d = 4; % The vector field is in R^d with d = 4
N = length(a)/d - 1; % N+1 is the number of Chebyshev coefficients per component

gamma = par(1);
beta = par(2);
rho = par(3);
tau = par(4);

i1 = (1:N+1); i2 = i1 + N+1; i3 = i2 + N+1; i4 = i3 + N+1;

a1 = a(i1); a2 = a(i2); a3 = a(i3); a4 = a(i4);
b2 = b(i2);

DF = intval(zeros(d*(N+1)));

%%%% We define some convolutions to built Dphi

a1a2 = rig_convolution([a1 a2]);
a1a3 = rig_convolution([a1 a3]);
a1a4 = rig_convolution([a1 a4]);
a2a4 = rig_convolution([a2 a4]);
a3a4 = rig_convolution([a3 a4]);
a4a4 = rig_convolution([a4 a4]);
a2b2 = rig_convolution([a2 b2]);
a3b2 = rig_convolution([a3 b2]);
a4b2 = rig_convolution([a4 b2]);

a1a2a2 = rig_convolution([a1 a2 a2]); a1a2a2 = a1a2a2(1:2*N+1);
a1a2a3 = rig_convolution([a1 a2 a3]); a1a2a3 = a1a2a3(1:2*N+1);
a2a2a3 = rig_convolution([a2 a2 a3]); a2a2a3 = a2a2a3(1:2*N+1);
a2a2b2 = rig_convolution([a2 a2 b2]); a2a2b2 = a2a2b2(1:2*N+1);
a2a3b2 = rig_convolution([a2 a3 b2]); a2a3b2 = a2a3b2(1:2*N+1);

phi1_a1 = -gamma*eye(N+1);

phi2_a1 = intval(zeros(N+1)); phi2_a2 = intval(zeros(N+1)); phi2_a3 = intval(zeros(N+1)); phi2_a4 = intval(zeros(N+1));
phi3_a1 = intval(zeros(N+1)); phi3_a3 = intval(zeros(N+1)); phi3_a4 = intval(zeros(N+1));
phi4_a1 = intval(zeros(N+1)); phi4_a4 = intval(zeros(N+1));

for ell=0:N
    
    k = (0:N);
    k1 = abs(k-ell)+1;
    k2 = k+ell+1;
    
    w = (1-isequal(ell,0)/2);
    
    phi2_a1(:,ell+1) = w*( -gamma*(a2a4(k1)+a2a4(k2)) + rho*gamma*(a2a2a3(k1)+a2a2a3(k2)) );
    phi2_a2(:,ell+1) = w*( -gamma*(a1a4(k1)+a1a4(k2)) + beta*(a4b2(k1)+a4b2(k2)) + 2*rho*gamma*(a1a2a3(k1)+a1a2a3(k2)) - 2*rho*beta*(a2a3b2(k1)+a2a3b2(k2)) );
    phi2_a3(:,ell+1) = w*( rho*gamma*(a1a2a2(k1)+a1a2a2(k2)) - rho*beta*(a2a2b2(k1)+a2a2b2(k2)) );
    phi2_a4(:,ell+1) = w*( -gamma*(a1a2(k1)+a1a2(k2)) + beta*(a2b2(k1)+a2b2(k2)) );
    
    phi3_a1(:,ell+1) = w*( -gamma*(rho-2)*(a3a4(k1)+a3a4(k2)) );
    phi3_a3(:,ell+1) = w*( -gamma*(rho-2)*(a1a4(k1)+a1a4(k2)) + beta*(rho-2)*(a4b2(k1)+a4b2(k2)) );
    phi3_a4(:,ell+1) = w*( -gamma*(rho-2)*(a1a3(k1)+a1a3(k2)) + beta*(rho-2)*(a3b2(k1)+a3b2(k2)) );
    
    phi4_a1(:,ell+1) = w*( gamma*(a4a4(k1)+a4a4(k2)) );
    phi4_a4(:,ell+1) = w*( 2*gamma*(a1a4(k1)+a1a4(k2)) - 2*beta*(a4b2(k1)+a4b2(k2)) );
    
end

T = intval(diag(ones(1,N),-1) - diag(ones(1,N),1)) ; T(1,2)=0;
Lambda_inv = intval(diag([1 1:N].^(-1)));
Id = intval(eye(N+1));

%%% F1_a1 %%%
DF(i1,i1) = Id - (tau/4)*Lambda_inv*T*phi1_a1;

%%% F2_a1 %%%
DF(i2,i1) = - (tau/4)*Lambda_inv*T*phi2_a1;

%%% F2_a2 %%%
DF(i2,i2) = Id - (tau/4)*Lambda_inv*T*phi2_a2;

%%% F2_a3 %%%
DF(i2,i3) = - (tau/4)*Lambda_inv*T*phi2_a3;

%%% F2_a4 %%%
DF(i2,i4) = - (tau/4)*Lambda_inv*T*phi2_a4;

%%% F3_a1 %%%
DF(i3,i1) = - (tau/4)*Lambda_inv*T*phi3_a1;

%%% F3_a3 %%%
DF(i3,i3) = Id - (tau/4)*Lambda_inv*T*phi3_a3;

%%% F3_a4 %%%
DF(i3,i4) = - (tau/4)*Lambda_inv*T*phi3_a4;

%%% F4_a1 %%%
DF(i4,i1) = - (tau/4)*Lambda_inv*T*phi4_a1;

%%% F3_a1 %%%
DF(i4,i4) = Id - (tau/4)*Lambda_inv*T*phi4_a4;

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% d_Fj_0/d_aj (j=1,2,3,4) %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

at_minus1 = intval([1 2*(-1).^(1:N)]');

DF(i1(1),i1) = at_minus1;
DF(i2(1),i2) = at_minus1;
DF(i3(1),i3) = at_minus1;
DF(i4(1),i4) = at_minus1;

end
